from arr2_epec_seg_ex import *
from gui.gui import GUI, QtCore, QtGui, QtWidgets, Qt, QPointF
import conversions
import read_input
import verify
import sys

if len(sys.argv) < 4:
    exit()

def r_ex(P: Polygon_2, v1) -> FT:
    max_x = max(P.vertices(), key=lambda p: p.x())
    return max_x.x() - v1.x()


def l_ex(P: Polygon_2, v1) -> FT:
    min_x = min(P.vertices(), key=lambda p: p.x())
    return v1.x() - min_x.x()


def t_ex(P: Polygon_2, v1) -> FT:
    max_y = max(P.vertices(), key=lambda p: p.y())
    return max_y.y() - v1.y()


def b_ex(P: Polygon_2, v1) -> FT:
    min_y = min(P.vertices(), key=lambda p: p.y())
    return v1.y() - min_y.y()


def run():
    rec: Polygon_2 = read_input.read_polygon(str(sys.argv[1]))

    P: Polygon_2 = read_input.read_polygon(str(sys.argv[2]))

    placements = read_input.read_result(str(sys.argv[3]))

    cos_a, sin_a = read_input.read_angle(str(sys.argv[4]))

    n = verify.count(rec, P, placements)
    if n >= 0:
        print("Polygons packed:", n)
    else:
        print("Invalid solution")
        #return


    copies = []
    for placement in placements:
        rt = Aff_transformation_2(Rotation(), placement[3], placement[2], FT(1))
        tt = Aff_transformation_2(Translation(), Vector_2(placement[0], placement[1]))
        transformed_vertices = [tt.transform(rt.transform(p)) for p in P.vertices()]
        transformed_P = Polygon_2(transformed_vertices)
        copies.append(transformed_P)

    # rational rotation approx example:
    # dir_x = FT(1)
    # dir_y = FT(1)
    # eps_num = FT(1)
    # eps_den = FT(1000)
    # sin_num = FT()
    # cos_num = FT()
    # denom = FT()
    # rational_rotation_approximation(dir_x, dir_y, sin_num, cos_num, denom, eps_num, eps_den)
    # assert (sin_num/denom*sin_num/denom + cos_num/denom*cos_num/denom == FT(1))

    assert (sin_a * sin_a + cos_a * cos_a == FT(1))
    rt = Aff_transformation_2(Rotation(), sin_a, cos_a, FT(1))
    rotated_P: Polygon_2 = Polygon_2([rt.transform(Point_2(p.x(), p.y())) for p in P.vertices()])
    v1 = rotated_P.vertex(0)
    minus_rotated_P = Polygon_2([Point_2(-p.x(), -p.y()) for p in rotated_P.vertices()])

    p0 = rec.vertex(0) + Vector_2(l_ex(rotated_P, v1), b_ex(rotated_P, v1))
    p1 = rec.vertex(1) + Vector_2(-r_ex(rotated_P, v1), b_ex(rotated_P, v1))
    p2 = rec.vertex(2) + Vector_2(-r_ex(rotated_P, v1), -t_ex(rotated_P, v1))
    p3 = rec.vertex(3) + Vector_2(l_ex(rotated_P, v1), -t_ex(rotated_P, v1))

    valid_rec = Polygon_2([p0, p1, p2, p3])

    PWH: Polygon_with_holes_2 = Polygon_with_holes_2(rec, copies)

    ms = []
    for hole in PWH.holes():
        ms.append(minkowski_sum_2(hole, minus_rotated_P))

    # Display
    # Rectangle
    gui.add_polygon(conversions.polygon_2_to_tuples_list(PWH.outer_boundary()), fill_color=Qt.gray)
    # Valid area regarding rectangle
    gui.add_polygon(conversions.polygon_2_to_tuples_list(valid_rec), fill_color=Qt.green)
    Q: Polygon_with_holes_2
    ps: Polygon_set_2 = Polygon_set_2()
    ps.join_polygons_with_holes(ms)
    ps.intersection(rec)
    pwh_lst = []
    ps.polygons_with_holes(pwh_lst)
    for Q in pwh_lst:
        # invalid area regarding already placed copies
        gui.add_polygon(conversions.polygon_2_to_tuples_list(Q.outer_boundary()), fill_color=Qt.blue)
        for hole in Q.holes():
            gui.add_polygon(conversions.polygon_2_to_tuples_list(hole), fill_color=Qt.green)

    # Add already placed copies of P
    for hole in PWH.holes():
        gui.add_polygon(conversions.polygon_2_to_tuples_list(hole), fill_color=Qt.red)

    tt = Aff_transformation_2(Translation(), Vector_2(-r_ex(rotated_P, v1), FT(0)))
    transformed_vertices = [tt.transform(p) for p in rotated_P.vertices()]
    rotated_P = Polygon_2(transformed_vertices)
    gui.add_polygon(conversions.polygon_2_to_tuples_list(rotated_P), fill_color=Qt.transparent)
    gui.add_disc(0.03, *conversions.point_2_to_xy(rotated_P.vertex(0)))
    print("In red - already placed copies of P")
    print("In green - valid positions for placing the first vertex of P in the provided orientation")
    print("In gray - invalid positions due to collision with the bounding rectangle")
    print("In blue - invalid positions due to collisions with other placements of P")
    print("You can zoom the view in/out using the +/- keys on your keyboard, respectively")
if __name__ == "__main__":
    import sys

    app = QtWidgets.QApplication(sys.argv)
    gui = GUI()
    gui.set_program_name("Polygon Packing")
    run()
    gui.MainWindow.show()
    sys.exit(app.exec_())
